
# Common subroutines
# Copyright (c) 2002-2004 Josef Zlomek
#
# This file is part of Fast File Search (FFS).
#
# FFS is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA,
# or download it from http://www.gnu.org/licenses/gpl.html

use strict;

sub read_config
{
	my ($c) = @_;

	$c->{CONFIG_FILE} = "/etc/ffsearch.conf";
	open (IN, "<$c->{CONFIG_FILE}") || die "Could not open $c->{CONFIG_FILE}\nReason: $!\n";

	$c->{VERSION} = "1.1.12";

	# semaphore indexes (for ffsearch.pl)
	$c->{SEM_LOG} = 0;

	# various values needed by crawler
	$c->{DB_BATCH} = 250;	# max joined insert/delete statements
	$c->{MONTHS} = {'Jan'=>'01', 'Feb'=>'02', 'Mar'=>'03',
		'Apr'=>'04', 'May'=>'05','Jun'=>'06',
		'Jul'=>'07', 'Aug'=>'08', 'Sep'=>'09',
		'Oct'=>'10','Nov'=>'11', 'Dec'=>'12'};

	# host types
	$c->{SMB} = 1 << 0;
	$c->{FTP} = 1 << 1;

	# host flags
	$c->{HOST_DELETE}  = 1 << 0;	# delete host
	$c->{HOST_PURGE}   = 1 << 1;	# purge host
	#$c->{HOST_SCANNED} = 1 << 2;	# no longer used, host was scanned
	$c->{HOST_ONLINE}  = 1 << 4;	# host was online during last online check
	# protocol specific flags
	# SMB:
	$c->{HOST_MANUAL_CHANGES} = 1 << 3;
	# FTP:
	$c->{HOST_PASSIVE_MODE} = 1 << 3;
	$c->{HOST_LS_RECURSIVE} = 1 << 5;

	# admin rights
	$c->{RIGHT_FTP_EDIT}  = 1 << 0;
	$c->{RIGHT_FTP_PURGE} = 1 << 1;
	$c->{RIGHT_SMB_EDIT}  = 1 << 2;
	$c->{RIGHT_SMB_PURGE} = 1 << 3;
	$c->{RIGHT_COMMENTS}  = 1 << 4;
	$c->{RIGHT_FTP_ADD}   = 1 << 5;
	$c->{RIGHT_SMB_ADD}   = 1 << 6;

	# these variables must be defined in config file
	$c->{LOG_FILE} =
	$c->{LOCK_FILE} =
	$c->{DB_NAME} =
	$c->{DB_LOGIN} =
	$c->{DB_PASSWORD} =
	$c->{SMB_PATH} = undef;

	# these variables are optional
	$c->{SMB_DISABLE} = "no";
	$c->{FTP_DISABLE} = "no";
	$c->{DEBUG_LEVEL} = 1;
	$c->{TIMEOUT_LOG} = 185;
	$c->{TIMEOUT_HISTORY} = 90;
	$c->{MAX_TEMP_FILE_SIZE} = "100M";
	$c->{MAX_DIR_DEPTH} = 20;
	$c->{CONVERT_TO_UPPER_CASE} = "yes";
	$c->{ONLINE_CHECK_SMB} = "nmblookup";
	$c->{ONLINE_CHECK_FTP} = "connect:2:10";
	$c->{ONLINE_CHECK} = "connect";		# for backwards compatibility
	$c->{ONLINE_CHECK_TIMEOUT} = 2;		# for backwards compatibility
	$c->{ONLINE_CHECK_PARALLEL} = 10;	# for backwards compatibility
	$c->{DB_HOST} = "";
	$c->{DB_PORT} = 0;
	$c->{MDB_HOST} = "";
	$c->{MDB_PORT} = 0;
	$c->{MDB_NAME} = "";
	$c->{MDB_LOGIN} = "";
	$c->{MDB_PASSWORD} = "";
	$c->{SMB_SERVER} = "";
	$c->{SMB_WORKGROUP} = "";
	$c->{SMB_USER} = "NOBODY";
	$c->{SMB_CONF} = "";
	$c->{SMB_PASSWORD} = "";
	$c->{SMB_CRAWL_HIDDEN} = "no";
	$c->{SMB_EXPIRE} = 5;
	$c->{SMB_PARALLEL} = 1;
	$c->{SMB_EXCLUDE_SHARES} = ();
	$c->{FTP_ANON_PASSWD} = "ffsearch\@host.com";
	$c->{FTP_TIMEOUT} = 30;
	$c->{FTP_EXPIRE} = 10;
	$c->{FTP_PARALLEL} = 1;
	$c->{FTP_MAX_DIR_DEPTH} = 20;

	# these variables are not used by perl scripts but by WWW interface
	# define some values so that we do not write "invalid entry" warning
	$c->{LANGUAGE} = "en";
	$c->{HIDE_LANGUAGES} = "";
	$c->{HISTORY_COUNT} = 10;
	$c->{BACKUP_URL} = "";
	$c->{DENY_SLOW_QUERIES} = "yes";
	$c->{FAST_QUERY_MINSIZE} = "10m";
	$c->{FAST_QUERY_MAXSIZE} = "disabled";
	$c->{LOGGING_ENABLE} = "yes";
	$c->{LOGGING_DNS_LOOKUP} = "no";
	$c->{USE_IP_ADDRESS_IN_LINK} = "no";
	$c->{SHOW_LOGIN_PASS} = "yes";
	$c->{HOST_FILE_SIZE} = "yes";
	$c->{SHARE_FILE_SIZE} = "yes";
	$c->{SHOW_LAST_CHANGE} = "yes";
	$c->{ADD_FTP_HOST_ENABLED} = "yes";
	$c->{ADD_SMB_HOST_ENABLED} = "yes";
	$c->{HOST_EDITABLE_SELF} = "yes";
	$c->{HOST_EDITABLE_OTHER} = "no";

	# read the config file
	while (<IN>)
	{
		next if (/^#/ || /^$/);
		s/#.*$//;
		if (/^\s*(\S+)\s+(.*)$/)
		{
			my $var = uc ($1);
			if (exists $c->{$var})
			{
				$c->{$var} = $2;
				$c->{$var} =~ s/\x00.*//;
				$c->{$var} =~ s/^\s+//;
				$c->{$var} =~ s/\s+$//;
				$c->{"__$var"} = 1;
			}
			else
			{
				print "Invalid entry $var\n";
			}
		}
	}
	close (IN);
	$c->{DB_HOST} = "localhost" if (!$c->{DB_HOST});
	$c->{DB_PORT} = int ($c->{DB_PORT});
	$c->{DB_PORT} = 3306 if (!$c->{DB_PORT});
	$c->{DB} = "DBI:mysql:" . $c->{DB_NAME} . ";host=" . $c->{DB_HOST} . ";port=" . $c->{DB_PORT};

	$c->{MDB} = "";
	my $mdb_defined = $c->{__MDB_NAME} + $c->{__MDB_LOGIN} + $c->{__MDB_PASSWORD};
	if ($mdb_defined == 3)
	{
		$c->{MDB_HOST} = "localhost" if (!$c->{MDB_HOST});
		$c->{MDB_PORT} = int ($c->{MDB_PORT});
		$c->{MDB_PORT} = 3306 if (!$c->{MDB_PORT});
		$c->{MDB} = "DBI:mysql:" . $c->{MDB_NAME} . ";host=" . $c->{MDB_HOST} . ";port=" . $c->{MDB_PORT};
	}
	elsif ($mdb_defined > 0)
	{
		print "Master database configuration is not complete,\n  at least MDB_NAME, MDB_LOGIN and MDB_PASSWORD have to be defined.\n";
	}
	else
	{
		if ($c->{__MDB_HOST})
		{
			print "MDB_HOST is defined but MDB_NAME, MDB_LOGIN and MDB_PASSWORD are not defined\n";
		}
		if ($c->{__MDB_PORT})
		{
			print "MDB_PORT is defined but MDB_NAME, MDB_LOGIN and MDB_PASSWORD are not defined\n";
		}
	}
	if (!$c->{MDB})
	{
		$c->{MDB} = $c->{DB};
		$c->{MDB_LOGIN} = $c->{DB_LOGIN};
		$c->{MDB_PASSWORD} = $c->{DB_PASSWORD};
	}

	$c->{SMB_PATH_UNQUOTED} = $c->{SMB_PATH};
	$c->{SMB_PATH} = &shell_quote ($c->{SMB_PATH});
	$c->{SMB_ACCPARAM} = '-U ' . &shell_quote ($c->{SMB_USER} . '%' . $c->{SMB_PASSWORD});
	$c->{SMB_CONF} = ' -s ' . &shell_quote ($c->{SMB_CONF}) if ($c->{SMB_CONF});
	$c->{SMB_DISABLE} = lc ($c->{SMB_DISABLE});
	$c->{FTP_DISABLE} = lc ($c->{FTP_DISABLE});
	$c->{CONVERT_TO_UPPER_CASE} = lc ($c->{CONVERT_TO_UPPER_CASE});
	$c->{SMB_SERVER} = uc ($c->{SMB_SERVER});
	$c->{SMB_WORKGROUP} = uc ($c->{SMB_WORKGROUP});
	$c->{SMB_CRAWL_HIDDEN} = lc ($c->{SMB_CRAWL_HIDDEN});
	$c->{LANGUAGE} = lc ($c->{LANGUAGE});
	$c->{HIDE_LANGUAGES} = lc ($c->{HIDE_LANGUAGES});
	$c->{DENY_SLOW_QUERIES} = lc ($c->{DENY_SLOW_QUERIES});
	$c->{LOGGING_ENABLE} = lc ($c->{LOGGING_ENABLE});
	$c->{LOGGING_DNS_LOOKUP} = lc ($c->{LOGGING_DNS_LOOKUP});
	$c->{USE_IP_ADDRESS_IN_LINK} = lc ($c->{USE_IP_ADDRESS_IN_LINK});
	$c->{ONLINE_CHECK_SMB} = lc ($c->{ONLINE_CHECK_SMB});
	$c->{ONLINE_CHECK_FTP} = lc ($c->{ONLINE_CHECK_FTP});
	$c->{ONLINE_CHECK} = lc ($c->{ONLINE_CHECK});				# for backwards compatibility
	$c->{ONLINE_CHECK_PARALLEL} = int ($c->{ONLINE_CHECK_PARALLEL});	# for backwards compatibility
	$c->{SHOW_LOGIN_PASS} = lc ($c->{SHOW_LOGIN_PASS});
	$c->{HOST_FILE_SIZE} = lc ($c->{HOST_FILE_SIZE});
	$c->{SHARE_FILE_SIZE} = lc ($c->{SHARE_FILE_SIZE});
	$c->{SHOW_LAST_CHANGE} = lc ($c->{SHOW_LAST_CHANGE});
	$c->{ADD_FTP_HOST_ENABLED} = lc ($c->{ADD_FTP_HOST_ENABLED});
	$c->{ADD_SMB_HOST_ENABLED} = lc ($c->{ADD_SMB_HOST_ENABLED});
	$c->{HOST_EDITABLE_SELF} = lc ($c->{HOST_EDITABLE_SELF});
	$c->{HOST_EDITABLE_OTHER} = lc ($c->{HOST_EDITABLE_OTHER});
	$c->{TIMEOUT_LOG} = int ($c->{TIMEOUT_LOG});
	$c->{TIMEOUT_HISTORY} = int ($c->{TIMEOUT_HISTORY});
	$c->{MAX_DIR_DEPTH} = int ($c->{MAX_DIR_DEPTH});
	$c->{FTP_TIMEOUT} = int ($c->{FTP_TIMEOUT});
	$c->{SMB_EXPIRE} = int ($c->{SMB_EXPIRE});
	$c->{FTP_EXPIRE} = int ($c->{FTP_EXPIRE});
	$c->{SMB_PARALLEL} = int ($c->{SMB_PARALLEL});
	$c->{FTP_PARALLEL} = int ($c->{FTP_PARALLEL});
	$c->{FTP_MAX_DIR_DEPTH} = int ($c->{FTP_MAX_DIR_DEPTH});
	$c->{SMB_EXCLUDE_SHARES} = [ split (/\s*,\s*/, $c->{SMB_EXCLUDE_SHARES}) ];
	foreach (@{$c->{SMB_EXCLUDE_SHARES}}){
		s/^\s*"//;
		s/"\s*$//;
		$_ = qr/$_/i;
	}

	# check config
	my %known_languages = (
		"bg" => "Bulgarian",
		"cs" => "Czech",
		"de" => "German",
		"en" => "English",
		"es" => "Spanish",
		"fr" => "French",
		"hu" => "Hungarian",
		"lt" => "Lithuanian",
		"nb" => "Norwegian",
		"nl" => "Dutch",
		"pl" => "Polish",
		"ro" => "Romanian",
		"ru" => "Russian",
		"sk" => "Slovak",
		"uk" => "Ukrainian",
		"zh" => "Chinese",
		);
	my $error = 0;
	$error |= &check_yes_no ('SMB_DISABLE', $c->{SMB_DISABLE});
	$error |= &check_yes_no ('FTP_DISABLE', $c->{FTP_DISABLE});
	$error |= &check_yes_no ('CONVERT_TO_UPPER_CASE', $c->{CONVERT_TO_UPPER_CASE});
	$error |= &check_yes_no ('SMB_CRAWL_HIDDEN', $c->{SMB_CRAWL_HIDDEN});
	$error |= &check_yes_no ('DENY_SLOW_QUERIES', $c->{DENY_SLOW_QUERIES});
	$error |= &check_yes_no ('LOGGING_ENABLE', $c->{LOGGING_ENABLE});
	$error |= &check_yes_no ('LOGGING_DNS_LOOKUP', $c->{LOGGING_DNS_LOOKUP});
	$error |= &check_yes_no ('USE_IP_ADDRESS_IN_LINK', $c->{USE_IP_ADDRESS_IN_LINK});
	$error |= &check_yes_no ('SHOW_LOGIN_PASS', $c->{SHOW_LOGIN_PASS});
	$error |= &check_yes_no ('HOST_FILE_SIZE', $c->{HOST_FILE_SIZE});
	$error |= &check_yes_no ('SHARE_FILE_SIZE', $c->{SHARE_FILE_SIZE});
	$error |= &check_yes_no ('SHOW_LAST_CHANGE', $c->{SHOW_LAST_CHANGE});
	$error |= &check_yes_no ('ADD_FTP_HOST_ENABLED', $c->{ADD_FTP_HOST_ENABLED});
	$error |= &check_yes_no ('ADD_SMB_HOST_ENABLED', $c->{ADD_SMB_HOST_ENABLED});
	$error |= &check_yes_no ('HOST_EDITABLE_SELF', $c->{HOST_EDITABLE_SELF});
	$error |= &check_yes_no ('HOST_EDITABLE_OTHER', $c->{HOST_EDITABLE_OTHER});
	$c->{TIMEOUT_LOG} = 185 if ($c->{TIMEOUT_LOG} < 1);
	$c->{TIMEOUT_HISTORY} = 90 if ($c->{TIMEOUT_LOG} < 1);
	$c->{MAX_DIR_DEPTH} = 1 if ($c->{MAX_DIR_DEPTH} < 1);
	$c->{FTP_TIMEOUT} = 30 if ($c->{FTP_TIMEOUT} < 1);
	$c->{SMB_EXPIRE} = 5 if ($c->{SMB_EXPIRE} < 1);
	$c->{FTP_EXPIRE} = 10 if ($c->{FTP_EXPIRE} < 1);
	$c->{SMB_PARALLEL} = 1 if ($c->{SMB_PARALLEL} < 1);
	$c->{FTP_PARALLEL} = 1 if ($c->{FTP_PARALLEL} < 1);

	my $print_known_languages = 0;
	if (!$known_languages{$c->{LANGUAGE}})
	{
		print "Unknown language: '$c->{LANGUAGE}'\n";
		$print_known_languages = 1;
	}
	if ($c->{HIDE_LANGUAGES})
	{
		my @langs = split (/\s*,\s*/, $c->{HIDE_LANGUAGES});
		foreach (@langs)
		{
			if ($_ ne '' && $_ ne 'all' && !$known_languages{$_})
			{
				print "Unknown language: '$_'\n";
				$print_known_languages = 1;
			}
		}
	}
	if ($print_known_languages)
	{
		print "Known languages: ";
		foreach (sort keys %known_languages)
		{
			print "'$_' ($known_languages{$_}), ";
		}
		print "\n";
	}
	if ($c->{FAST_QUERY_MINSIZE} !~ /^\s*(\d+\s*[kKmMgG]?|disabled?)\s*$/)
	{
		print "FAST_QUERY_MINSIZE must be either a number (optionally) followed by\n  'k','K','m','M','g' or 'G';   or a word 'disabled'.\n";
	}
	if ($c->{FAST_QUERY_MAXSIZE} !~ /^\s*(\d+\s*[kKmMgG]?|disabled?)\s*$/)
	{
		print "FAST_QUERY_MAXSIZE must be either a number (optionally) followed by\n  'k','K','m','M','g' or 'G';   or a word 'disabled'.\n";
	}

	# For backwards compatibility
	if ($c->{__ONLINE_CHECK})
	{
		print "ONLINE_CHECK is deprecated.\n";
		print "  Use ONLINE_CHECK_SMB type:timeout:parallel\n";
		print "  and ONLINE_CHECK_FTP type:timeout:parallel\n";
		if ($c->{ONLINE_CHECK} ne 'ping' && $c->{ONLINE_CHECK} ne 'connect')
		{
			$c->{ONLINE_CHECK} = 'connect';
		}
	}
	if ($c->{__ONLINE_CHECK_PARALLEL})
	{
		print "ONLINE_CHECK_PARALLEL is deprecated.\n";
		print "  Use ONLINE_CHECK_SMB type:timeout:parallel\n";
		print "  and ONLINE_CHECK_FTP type:timeout:parallel\n";
		if ($c->{ONLINE_CHECK_PARALLEL} <= 0)
		{
			$c->{ONLINE_CHECK_PARALLEL} = 1 if ($c->{ONLINE_CHECK} eq 'ping');
			$c->{ONLINE_CHECK_PARALLEL} = 16 if ($c->{ONLINE_CHECK} eq 'connect');
		}
	}
	if ($c->{__ONLINE_CHECK_TIMEOUT})
	{
		print "ONLINE_CHECK_TIMEOUT is deprecated.\n";
		print "  Use ONLINE_CHECK_SMB type:timeout:parallel\n";
		print "  and ONLINE_CHECK_FTP type:timeout:parallel\n";
		if ($c->{ONLINE_CHECK_TIMEOUT} !~ /^(\d+|\d+\.\d*|\d*\.\d+)$/)
		{
			print "ONLINE_CHECK_TIMEOUT must be a (floating-point) number.\n";
			if ($c->{ONLINE_CHECK_TIMEOUT} =~ /^(\d+|\d+\.\d*|\d*\.\d+)/)
			{
				$c->{ONLINE_CHECK_TIMEOUT} = $1;
			}
			else
			{
				if ($c->{ONLINE_CHECK_PARALLEL} <= 4)
				{
					$c->{ONLINE_CHECK_TIMEOUT} = 1;
				}
				else
				{
					$c->{ONLINE_CHECK_TIMEOUT} = $c->{ONLINE_CHECK_PARALLEL} / 4;
				}
			}
		}
		if ($c->{ONLINE_CHECK} eq 'ping')
		{
			$c->{ONLINE_CHECK_TIMEOUT} = int ($c->{ONLINE_CHECK_TIMEOUT} + 0.999999999);
		}
		$c->{ONLINE_CHECK_TIMEOUT} = 1 if ($c->{ONLINE_CHECK_TIMEOUT} == 0);
	}
	if ($c->{__FTP_MAX_DIR_DEPTH})
	{
		print "FTP_MAX_DIR_DEPTH is deprecated, use MAX_DIR_DEPTH instead.\n";
		if (!$c->{__MAX_DIR_DEPTH} && $c->{FTP_MAX_DIR_DEPTH} >= 1) {
			$c->{MAX_DIR_DEPTH} = $c->{FTP_MAX_DIR_DEPTH};
		}
	}

	&parse_online_check ($c, "SMB");
	&parse_online_check ($c, "FTP");

	my $size = &get_size ($c->{MAX_TEMP_FILE_SIZE}, -1);
	if ($size == -1)
	{
		print $c->{MAX_TEMP_FILE_SIZE}, " is not a valid size, must be a number\n  (optionally) followed by 'k','K','m','M','g' or 'G'.\n";
		$size = 0;
		$error = 1;
	}
	$c->{MAX_TEMP_FILE_SIZE} = $size;

	foreach (keys %$c)
	{
		if (!defined ($c->{$_}))
		{
			print "read_config: $_ must be defined\n";
			$error = 1;
		}
	}

	if ($error)
	{
		print "Configuration is not complete. Please check $c->{CONFIG_FILE}.\n";
		exit 1;
	}
}

sub parse_online_check
{
	my ($c, $proto) = @_;

	my $lc_proto = lc ($proto);

	if ($proto eq "SMB" && $c->{"ONLINE_CHECK_" . $proto} =~ /^nmblookup\s*$/)
	{
		$c->{"ONLINE_CHECK_" . $proto} = "nmblookup";
	}
	elsif ($c->{"ONLINE_CHECK_" . $proto} =~ /^ping:(\d+|\d+\.\d*|\d*\.\d+)\s*$/)
	{
		$c->{"ONLINE_CHECK_" . $proto . "_TIMEOUT"} = int ($1 + 0.999999999);
		$c->{"ONLINE_CHECK_" . $proto . "_TIMEOUT"} = 1 if ($c->{"ONLINE_CHECK_" . $proto . "_TIMEOUT"} == 0);
		$c->{"ONLINE_CHECK_" . $proto} = "ping";
	}
	elsif ($c->{"ONLINE_CHECK_" . $proto} =~ /^connect:(\d+|\d+\.\d*|\d*\.\d+):(\d+)\s*$/)
	{
		$c->{"ONLINE_CHECK_" . $proto . "_TIMEOUT"} = $1;
		$c->{"ONLINE_CHECK_" . $proto . "_TIMEOUT"} = 1 if ($c->{"ONLINE_CHECK_" . $proto . "_TIMEOUT"} == 0);
		$c->{"ONLINE_CHECK_" . $proto . "_PARALLEL"} = $2;
		$c->{"ONLINE_CHECK_" . $proto . "_PARALLEL"} = 1 if ($c->{"ONLINE_CHECK_" . $proto . "_PARALLEL"} == 0);
		$c->{"ONLINE_CHECK_" . $proto} = "connect";
	}
	elsif ($c->{"__ONLINE_CHECK_" . $proto})
	{
		print "Incorrect ONLINE_CHECK_$proto. It must be one of the following:\n";
		print "  online_check_$lc_proto nmblookup\n" if ($proto eq "SMB");
		print "  online_check_$lc_proto ping:TIMEOUT\n";
		print "  online_check_$lc_proto connect:TIMEOUT:PARALLEL\n";
		print "where\n";
		print "  TIMEOUT is a positive floating-point number\n";
		print "  PARALLEL is a positive integer number\n";
	}
	elsif ($c->{__ONLINE_CHECK})	# backwards compatibility
	{
		$c->{"ONLINE_CHECK_" . $proto} = $c->{ONLINE_CHECK};
		$c->{"ONLINE_CHECK_" . $proto . "_TIMEOUT"} = $c->{ONLINE_CHECK_TIMEOUT};
		$c->{"ONLINE_CHECK_" . $proto . "_PARALLEL"} = $c->{ONLINE_CHECK_PARALLEL};
	}
	else	# default values
	{
		if ($proto eq "SMB")
		{
			$c->{ONLINE_CHECK_SMB} = "nmblookup";
		}
		else
		{
			$c->{ONLINE_CHECK_FTP} = "connect";
			$c->{ONLINE_CHECK_FTP_TIMEOUT} = "2";
			$c->{ONLINE_CHECK_FTP_PARALLEL} = "10";
		}
	}
}

sub check_yes_no
{
	my ($key, $value) = @_;

	if ($value ne 'yes' && $value ne 'no')
	{
		print "read_config: $key must have value yes or no, current value is $value\n";
		return 1;
	}
	return 0;
}

sub get_size
{
	my ($str, $default) = @_;

	my $size = $default;
	if ($str =~ /^\s*(\d+)\s*([kKmMgG]?)\s*$/)
	{
		$size = $1;
		$size *= 1024 * 1024 * 1024 if ($2 eq 'G');
		$size *= 1024 * 1024 if ($2 eq 'M');
		$size *= 1024 if ($2 eq 'K');
		$size *= 1000 * 1000 * 1000 if ($2 eq 'g');
		$size *= 1000 * 1000 if ($2 eq 'm');
		$size *= 1000 if ($2 eq 'k');
	}
	return $size;
}

sub is_invalid_IP
{
	my ($ip) = @_;
	if ($ip =~ /^([1-9]\d{0,2})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/)
	{
		return 0 if ($1 <= 255 && $2 <= 255 && $3 <= 255 && $4 <= 255);
	}
	return 1;
}

sub db_connect
{
	my ($c, $mdb, $db, $error_print) = @_;

	if (!$error_print)
	{
		$error_print = sub { print STDERR $_[0], "\n"; };
	}

	if ($mdb)
	{
		undef ($$mdb);
		if (!$db || $c->{MDB} && ($c->{MDB} ne $c->{DB} || $c->{MDB_LOGIN} ne $c->{DB_LOGIN}))
		{
			$$mdb = DBI->connect ($c->{MDB}, $c->{MDB_LOGIN}, $c->{MDB_PASSWORD}, {PrintError => 0});
			if (!defined ($$mdb))
			{
				&$error_print ("Could not connect to master database, reason: $DBI::errstr", 0);
			}
		}
	}
	if ($db)
	{
		$$db = DBI->connect ($c->{DB}, $c->{DB_LOGIN}, $c->{DB_PASSWORD}, {PrintError => 0});
		if (!defined ($$db))
		{
			&$error_print ("Could not connect to database, reason: $DBI::errstr", 0);
		}
		$$mdb = $$db if ($mdb && !$$mdb);
	}
}

sub db_disconnect
{
	my ($mdb, $db) = @_;
	$mdb->disconnect () if ($mdb && $mdb != $db);
	$db->disconnect () if ($db);
}

sub echo_on
{
	system ("stty echo");
}

sub echo_off
{
	system ("stty -echo");
}

sub ask
{
	my ($text, $default) = @_;
	my $str;

	print $text;
	chomp ($str = <STDIN>);
	$str = $default if ($str =~ /^\r?$/);
	return $str;
}

# quote string so that it could be passed to shell
sub shell_quote
{
	my ($str) = @_;
	$str =~ s/'/'\\''/g;
	return "'$str'";
}

# quote string in scripts exporting data while upgrade
sub quote
{
	my ($tmp) = @_;
	$tmp =~ s/(\\|\')/\\$1/g;
	$tmp =~ s/\n/\\n/g;
	return "'$tmp'";
}

our $import_num = "(\\d+)";
our $import_nnum = "(-?\\d+)";
our $import_str0 = "'((?:[^\\\\']|\\\\[\\\\'n])*)'";
our $import_str1 = "'((?:[^\\\\']|\\\\[\\\\'n])+)'";

sub unquote_char
{
	my ($char) = @_;
	return "\n" if ($char eq 'n');
	return $char;
}

sub unquote
{
	my ($tmp) = @_;
	$tmp =~ s/\\(.)/&unquote_char($1)/ge;
	return $tmp;
}

sub create_tables
{
	my ($db) = @_;

	my $tables = $db->selectcol_arrayref ("SHOW TABLES");
	if (!$tables)
	{
		print "Error listing tables - reason: $DBI::errstr\n";
		return 1;
	}

	foreach (@$tables)
	{
		$db->do ("DROP TABLE $_");
	}

	$db->do ("CREATE TABLE host (
		HID INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
		IP CHAR(16) NOT NULL,
		HostType TINYINT UNSIGNED NOT NULL,
		ExpireCount TINYINT UNSIGNED NOT NULL,
		Period TINYINT UNSIGNED NOT NULL,
		Counter TINYINT UNSIGNED NOT NULL,
		ScanOrder TINYINT NOT NULL,
		Flags TINYINT UNSIGNED NOT NULL,
		LastScan DATETIME NOT NULL,
		TotalFileSize BIGINT NOT NULL DEFAULT '-1',
		KEY (TotalFileSize)
		)");

	$db->do ("CREATE TABLE ftp (
		HID INT UNSIGNED NOT NULL,
		HostName CHAR(64) NOT NULL,
		Login CHAR(32) NOT NULL,
		PassWord CHAR(64) NOT NULL,
		Port SMALLINT UNSIGNED NOT NULL,
		StartingDir CHAR(128) NOT NULL,
		FileList CHAR(128) NOT NULL,
		Comment CHAR(255) NOT NULL,
		EditableBy CHAR(64) NOT NULL,
		KEY (HID),
		KEY (HostName(8))
		)");

	$db->do ("CREATE TABLE smb (
		HID INT UNSIGNED NOT NULL,
		HostName CHAR(32) NOT NULL,
		WorkGroup CHAR(32) NOT NULL,
		Login CHAR(32) NOT NULL DEFAULT 'NOBODY',
		PassWord CHAR(64) NOT NULL,
		KEY (HID),
		KEY (HostName(8))
		)");

	$db->do ("CREATE TABLE share (
		SID INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
		HID INT UNSIGNED NOT NULL,
		ShareName CHAR(32) NOT NULL,
		Login CHAR(32) NOT NULL DEFAULT 'NOBODY',
		PassWord CHAR(64) NOT NULL,
		TotalFileSize BIGINT NOT NULL DEFAULT '-1',
		Flags TINYINT UNSIGNED NOT NULL,
		KEY (HID),
		KEY (ShareName(8)),
		KEY (TotalFileSize)
		)");

	$db->do ("CREATE TABLE path (
		PID INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
		`Hash` INT UNSIGNED NOT NULL,
		PPID INT UNSIGNED NOT NULL,
		HID INT UNSIGNED NOT NULL,
		SID INT UNSIGNED NOT NULL,
		ShortName VARCHAR(255) NOT NULL,
		FullName VARCHAR(255) NOT NULL,
		KEY (`Hash`),
		KEY (PPID),
		KEY (HID),
		KEY (SID)
		)");

	$db->do ("CREATE TABLE file (
		PID INT UNSIGNED NOT NULL,
		FileName VARCHAR(255) NOT NULL,
		FileNameEnd CHAR(4) NOT NULL,
		DateAdded INT UNSIGNED NOT NULL,
		FileSize INT UNSIGNED NOT NULL,
		FileDate DATETIME NOT NULL,
		KEY (PID),
		KEY (FileName(8)),
		KEY (FileNameEnd),
		KEY (FileSize)
		)");

	$db->do ("CREATE TABLE status (
		LastChange CHAR(30),
		SMBHosts INT UNSIGNED NOT NULL,
		FTPHosts INT UNSIGNED NOT NULL,
		Directories INT UNSIGNED NOT NULL,
		Files INT UNSIGNED NOT NULL,
		FileSize BIGINT NOT NULL DEFAULT '-1',
		Queries INT UNSIGNED NOT NULL,
		UpdatingHost INT UNSIGNED NOT NULL
		)");
	$db->do ("INSERT INTO status (LastChange) VALUES ('Net not scanned yet')");

	$db->do ("CREATE TABLE log (
		LID INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
		Time INT UNSIGNED NOT NULL,
		Duration FLOAT NOT NULL,
		`Found` INT UNSIGNED NOT NULL,
		Hits INT UNSIGNED NOT NULL,
		Position INT UNSIGNED NOT NULL,
		Client VARCHAR(64) NOT NULL,
		SearchString VARCHAR(255) NOT NULL,
		Mode TINYINT UNSIGNED NOT NULL,
		HostType TINYINT UNSIGNED NOT NULL,
		Flags TINYINT UNSIGNED NOT NULL,
		Date INT UNSIGNED NOT NULL,
		MinSize INT UNSIGNED NOT NULL,
		MaxSize INT UNSIGNED NOT NULL,
		KEY (Time)
		)");

	$db->do ("CREATE TABLE user (
		UID INT NOT NULL PRIMARY KEY,
		LastSearched INT UNSIGNED NOT NULL,
		Current TINYINT UNSIGNED NOT NULL,
		Save TINYINT UNSIGNED NOT NULL,
		KEY (LastSearched)
		)");

	$db->do ("CREATE TABLE history (
		UID INT NOT NULL,
		Position TINYINT UNSIGNED NOT NULL,
		SearchString VARCHAR(255) NOT NULL,
		Mode TINYINT UNSIGNED NOT NULL,
		HostType TINYINT UNSIGNED NOT NULL,
		Flags TINYINT UNSIGNED NOT NULL,
		Date TINYINT UNSIGNED NOT NULL,
		DateValue INT UNSIGNED NOT NULL,
		MinSize INT UNSIGNED NOT NULL,
		MaxSize INT UNSIGNED NOT NULL,
		Hits INT UNSIGNED NOT NULL,
		KEY (UID)
		)");

	$db->do ("CREATE TABLE comments (
		CID INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
		Time INT UNSIGNED NOT NULL,
		Name CHAR(40) NOT NULL,
		Email CHAR(60) NOT NULL,
		Comment TEXT NOT NULL,
		KEY (Time)
		)");

	$db->do ("CREATE TABLE admin (
		AID INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
		Login CHAR(32) NOT NULL,
		PassWord CHAR(32) NOT NULL,
		Name CHAR(32) NOT NULL,
		WWW CHAR(255) NOT NULL,
		Email CHAR(64) NOT NULL,
		Rights INT UNSIGNED NOT NULL,
		LoginTime INT UNSIGNED NOT NULL,
		LoginIP CHAR(16) NOT NULL,
		KEY (Login(8))
		)");
}

# make this package return true
1;
